/*
 * Impedance of seven strand cable
 * Cf.
 *   J.E.L.Tweeddle,- Proximity effect in seven-strand cable
 *	Trans.Am.I.E.E. 46, p1148-1927
 * 2017-05-04 Kouichi Hirabayashi
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <complex.h>
#include "Complex.h"

#define PROFILE

#define COMPLEX double complex

typedef struct { COMPLEX z1; COMPLEX z2; } C2;

typedef struct { int n; C2 c; } PROF;

COMPLEX Solid(double, double, double, double);
COMPLEX Solid2(double, double, double, double);


PROF prof[10][50];	// An, Bn, Cn, ... Fn, Gn, Hn ..

// X1=I1 loss by skin effect and mutual inductance
// X2=I2 loss by skin effect and mutual inductance
// X3=I1 loss by proximity effect
// X4=I2 loss by proximity effect
double X1, X2, X3, X4;

double PI;

COMPLEX A0(double a, COMPLEX A)
{
  COMPLEX z;

  z = I * A * a;
  return 0.5 * z / Jn(1, z);
}

// An = An.z1 * I1 + An.z2 * I2

C2 An(int n, double a, double s, COMPLEX A)
{
  double d;
  COMPLEX z, z1, z2;
  C2 c;

  z = I * A * a;
  d = (double)n;
  c.z1 = c.z2 = (COMPLEX)0.0;
  z2 = pow(a / s, d) / (PI * a * a) * z / Jn(n - 1, z);
  z1 = z2 * (2.0 * cos(d * PI / 3.0)
	+ 2.0 / pow(sqrt(3.0), d) * cos(d * PI / 6.0)
	+ pow(2.0, -d));
  c.z1 += z1; c.z2 += z2;
  return c;
}

C2 Fn(int n, double a, double s, COMPLEX A)
{
  double d;
  COMPLEX z, z1, z2;
  C2 c;

  z1 = z2 = (COMPLEX)0.0;
#if 1
  if ((n % 6) == 0) {
	d = (double)n;
	z = I * A * a;
	z1 += 6.0 * z / Jn(n - 1, z) / (PI * a * a) * pow(a / s, d);
  }
#else
  z1 += z / Jn(n - 1, z) / (PI * a * a) * pow(a / s, d)
	* (cos(d * PI) + 2.0 * cos(2.0 * d * PI / 3.0)
		+ 2.0 * cos(d * PI / 3.0) + 1.0);
#endif
  c.z1 = z1; c.z2 = z2;
  return c;
}

// (n + m - 1)! / ((n - 1)! * m!)
//     m
//  = (n - 1 + k) / k
//   k = 1
double Fnm(int n, int m)
{
  int k;
  double v;
  v = 1.0;
  for (k = 1; k <= m; k++)
	v *= (double)(n - 1 + k) / (double)k;
  return v;
}

C2 Bn(int n, double a, double s, COMPLEX A)
{
  int m;
  double d, k, e1, e2, e01, e02, d1, d2;
  COMPLEX z, z1, z2;
  C2 c, c1, c2, s1, s2, s10, s20;

  z = I * A * a;
  d = (double)n;
  c.z1 = c.z2 = (COMPLEX)0.0;
  e01 = e02 = 0.0;
  for (m = 1; m <= 24; m++) {
	z1 = pow(a / s, (double)(m + n)) * Fnm(n, m)
		* Jn(m - 1, z) / Jn(n - 1, z);
	c1 = Fn(m, a, s, A);
	k = cos((double)m * PI);
	c.z1 += z1 * c1.z1 * k;
	c.z2 += z1 * c1.z2 * k;
	c2 = An(m, a, s, A);
	k = 2.0 * cos(PI * (double)(m - n) / 3.0)
		+ 2.0 / pow(sqrt(3.0), (double)(m + n))
			* cos(PI * (double)(m - n) / 6.0)
		+ pow(2.0, -(double)(m + n));
	c.z1 += z1 * c2.z1 * k;
	c.z2 += z1 * c2.z2 * k;
	if (m % 6 == 0) {
		e1 = cabs(c.z1);
		e2 = cabs(c.z2);
		if (m > 6) {
			d1 = fabs((e1 - e01) / e1);
			d2 = fabs((e2 - e02) / e2);
			e01 = e1;
			e02 = e2;
			if (d1 < 1e-4 && d2 < 1e-4)
				break;
		}
	}
  }
  return c;
}

C2 Gn(int n, double a, double s, COMPLEX A)
{
  int m;
  double d, k, e1, e2, e01, e02, d1, d2;
  COMPLEX z, z1, z2;
  C2 c, c1, c2, s1, s2, s10, s20;

  c.z1 = c.z2 = 0;
  if ((n % 6) != 0)
	return c;
  z = I * A * a;
  d = (double)n;
  k = 6.0;
  e01 = e02 = 0.0;
  for (m = 6; m <= 24; m += 6) {
	z1 = pow(a / s, (double)(m + n)) * Fnm(n, m)
		* Jn(m + 1, z) / Jn(n - 1, z) * k;
	c1 = An(m, a, s, A);
	c.z1 += c1.z1 * z1;
	c.z2 += c1.z2 * z1;
	e1 = cabs(c.z1);
	e2 = cabs(c.z2);
	d1 = fabs((e1 - e01) / e1);
	d2 = fabs((e2 - e02) / e2);
	e01 = e1;
	e02 = e2;
	if (d1 < 1e-4 && d2 < 1e-4)
		break;
  }
  return c;
}

C2 Cn(int n, double a, double s, COMPLEX A)
{
  int m;
  double d, k, e1, e2, e01, e02, d1, d2;
  COMPLEX z, z1, z2;
  C2 c, c1, c2, s1, s2, s10, s20;

  z = I * A * a;
  d = (double)n;
  c.z1 = c.z2 = (COMPLEX)0.0;
  e01 = e02 = 0.0;
  for (m = 1; m <= 24; m++) {
	z1 = pow(a / s, (double)(m + n)) * Fnm(n, m)
		* Jn(m - 1, z) / Jn(n - 1, z);
	c1 = Gn(m, a, s, A);
	k = cos((double)m * PI);
	c.z1 += c1.z1 * z1 * k;
	c.z2 += c1.z2 * z1 * k;
	c2 = Bn(m, a, s, A);
	k = 2.0 * cos(PI * (double)(m - n) / 3.0)
		+ 2.0 / pow(sqrt(3.0), (double)(m + n))
			* cos(PI * (double)(m - n) / 6.0)
		+ pow(2.0, -(double)(m + n));
	c.z1 += c2.z1 * z1 * k;
	c.z2 += c2.z2 * z1 * k;
	if (m % 6 == 0) {
		e1 = cabs(c.z1);
		e2 = cabs(c.z2);
		if (m > 6) {
			d1 = fabs((e1 - e01) / e1);
			d2 = fabs((e2 - e02) / e2);
			e01 = e1;
			e02 = e2;
			if (d1 < 1e-4 && d2 < 1e-4)
				break;
		}
	}
  }
  return c;
}

C2 Hn(int n, double a, double s, COMPLEX A)
{
  int m;
  double d, k, e1, e2, e01, e02, d1, d2;
  COMPLEX z, z1, z2;
  C2 c, c1, c2, s1, s2, s10, s20;

  c.z1 = c.z2 = 0;
  if ((n % 6) != 0)
	return c;
  z = I * A * a;
  d = (double)n;
  k = 6.0;
  e01 = e02 = 0.0;
  for (m = 6; m <= 24; m += 6) {
	z1 = pow(a / s, (double)(m + n)) * Fnm(n, m)
		* Jn(m + 1, z) / Jn(n - 1, z) * k;
	c1 = Bn(m, a, s, A);
	c.z1 += c1.z1 * z1;
	c.z2 += c1.z2 * z1;
	e1 = cabs(c.z1);
	e2 = cabs(c.z2);
	d1 = fabs((e1 - e01) / e1);
	d2 = fabs((e2 - e02) / e2);
	e01 = e1;
	e02 = e2;
	if (d1 < 1e-4 && d2 < 1e-4)
		break;
  }
  return c;
}

C2 Dn(int n, double a, double s, COMPLEX A)
{
  int m;
  double d, k, e1, e2, e01, e02, d1, d2;
  COMPLEX z, z1, z2;
  C2 c, c1, c2, s1, s2, s10, s20;

  z = I * A * a;
  d = (double)n;
  c.z1 = c.z2 = (COMPLEX)0.0;
  e01 = e02 = 0.0;
  for (m = 1; m <= 24; m++) {
	z1 = pow(a / s, (double)(m + n)) * Fnm(n, m)
		* Jn(m - 1, z) / Jn(n - 1, z);
	c1 = Hn(m, a, s, A);
	k = cos((double)m * PI);
	c.z1 += c1.z1 * z1 * k;
	c.z2 += c1.z2 * z1 * k;
	c2 = Cn(m, a, s, A);
	k = 2.0 * cos(PI * (double)(m - n) / 3.0)
		+ 2.0 / pow(sqrt(3.0), (double)(m + n))
			* cos(PI * (double)(m - n) / 6.0)
		+ pow(2.0, -(double)(m + n));
	c.z1 += c2.z1 * z1 * k;
	c.z2 += c2.z2 * z1 * k;
	if (m % 6 == 0) {
		e1 = cabs(c.z1);
		e2 = cabs(c.z2);
		if (m > 6) {
			d1 = fabs((e1 - e01) / e1);
			d2 = fabs((e2 - e02) / e2);
			e01 = e1;
			e02 = e2;
			if (d1 < 1e-4 && d2 < 1e-4)
				break;
		}
	}
  }
  return c;
}

C2 Ik(int n, double a, double s, COMPLEX A)
{
  int m;
  double d, k, e1, e2, e01, e02, d1, d2;
  COMPLEX z, z1, z2;
  C2 c, c1, c2, s1, s2, s10, s20;

  c.z1 = c.z2 = 0;
  if ((n % 6) != 0)
	return c;
  z = I * A * a;
  d = (double)n;
  k = 6.0;
  e01 = e02 = 0.0;
  for (m = 6; m <= 24; m += 6) {
	z1 = pow(a / s, (double)(m + n)) * Fnm(n, m)
		* Jn(m + 1, z) / Jn(n - 1, z) * k;
	c1 = Cn(m, a, s, A);
	c.z1 += c1.z1 * z1;
	c.z2 += c1.z2 * z1;
	e1 = cabs(c.z1);
	e2 = cabs(c.z2);
	d1 = fabs((e1 - e01) / e1);
	d2 = fabs((e2 - e02) / e2);
	e01 = e1;
	e02 = e2;
	if (d1 < 1e-4 && d2 < 1e-4)
		break;
  }
  return c;
}


#ifdef PROFILE
int profile()
{
  int i, j, k;

  for (i = 1; i < 24; i++) {
    printf("%2d", i);
#if 1
    for (j = 0; j < 3; j++)
	printf(" %8.2e", cabs(prof[j][i].c.z1 + prof[j][i].c.z2));
//	printf(" %8.2e/%2d", cabs(prof[j][i].c.z1 + prof[j][i].c.z2), prof[j][i].n);
#else
    for (j = 1; j < 5; j++)
	printf(" %8.2e/%8.2e", cabs(prof[j][i].c.z1), cabs(prof[j][i].c.z2));
    for (j = 5; j < 9; j++)
	printf(" %8.2e/%8.2e", cabs(prof[j][i].c.z1), cabs(prof[j][i].c.z2));
#endif
    printf("\n");
  }
 return 0;
}
#endif

C2 Mn(int n, double a, double s, COMPLEX A)
{
  static C2 (*fun[])(int, double, double, COMPLEX) = {
        (void *)0, An, Bn, Cn, Dn };
  int i;
  C2 c, c1;

  c.z1 = c.z2 = (COMPLEX)0.0;
  for (i = 1; i <= 3; i++) {
	c1 = (*fun[i])(n, a, s, A);
	c.z1 += c1.z1; c.z2 += c1.z2;
  }
  return c;
}

C2 Nn(int n, double a, double s, COMPLEX A)
{
  static C2 (*fun[])(int, double, double, COMPLEX) = {
        (void *)0, Fn, Gn, Hn, Ik };
  int i;
  C2 c, c1;

  c.z1 = c.z2 = (COMPLEX)0.0;

  if (n % 6 == 0) {
	for (i = 1; i <= 3; i++) {
		c1 = (*fun[i])(n, a, s, A);
		c.z1 += c1.z1; c.z2 += c1.z2;
	}
  }
  return c;
}

// Gauss to SI unit syustem coversion factor

#define U	1e-7		// 0/(4*PI)
#define U1	1.11259e-10	// 4*PI*0, 0=4*PI*1e7/(4*PI*c*c), c=2.998e8
#define U2	1.25664e-6	// 0 = 4e-7*PI
#define U3	1.05479e-5	// sqrt(4*PI*0)

// Ea() = c1.z1 * I1 + c1.z2 * I2

C2 Ea(double a, double s, double w, double u, double g)
{
  int n;
  C2 c, c1, c2;
  COMPLEX  A, z, z1, z2;
  double d, e0, e, k, p;

  A = csqrt(I * w * u * g);
  z = I * A * a;
  c.z1 = c.z2 = 0;
  z1 = A0(a, A) / (PI * a * a * g) * Jn(0, z);
  z2 = (COMPLEX)0.0;
  // mutual inductance
  z1 += - I * w * U * 2.0 * log(6.0 * a * pow(s, 5.0));
  z2 += - I * w * U * 2.0 * log(s);
  c.z1 += z1; c.z2 += z2;
  // proximity effect by another outer conductors
  c1.z1 = c1.z2 = (COMPLEX)0;
  e0 = 0.0;
  for (n = 1; n <= 24; n++) {
	d = (double)n;
	z2 = pow(a / s, d) / d * cos(d * PI) * Jn(n + 1, z) / z;
	c2 = Nn(n, a, s, A);
	c1.z1 += c2.z1 * z2; c1.z2 += c2.z2 * z2;
#ifdef PROFILE
	prof[0][n].c = c1; prof[2][n].n = n;
#endif
	if (n % 6 == 0) {
		e = cabs(c.z1) + cabs(c.z2);
		if (fabs((e - e0) / e) < 1e-4)
			break;
		e0 = e;
	}
  }
  z1 = I * w * U1 * 2.0 * PI * a * a;
  c.z1 += c1.z1 * z1; c.z2 += c1.z2 * z1;

  // proximity effect by inner conductor
  c1.z1 = c1.z2 = (COMPLEX)0;
  e0 = 0.0;
  for (n = 1; n <= 12; n++) {
	d = (double)n;
	z2 = Jn(n + 1, z) / z * pow(a / s, d) / d
		* (2.0 * cos(d * PI / 3.0)
		+ 2.0 / pow(sqrt(3.0), d) * cos(d * PI / 6.0)
		+ pow(2.0, -d));
	c2 = Mn(n, a, s, A);
	c1.z1 += c2.z1 * z2; c1.z2 += c2.z2 * z2;
#ifdef PROFILE
	prof[1][n].c = c1; prof[1][n].n = n;
#endif
	e = cabs(c1.z1) + cabs(c1.z2);
	if (fabs((e - e0) / e) < 1e-4)
		break;
	e0 = e;
  }
  z1 = I * w * U1 * 2.0 * PI * a * a;
  c.z1 += c1.z1 * z1; c.z2 += c1.z2 * z1;
  return c;
}

// Eg() = c2.z1 * I1 + c2.z2 * I2

C2 Eg(double a, double s, double w, double u, double g)
{
  int n;
  C2 c, c1, c2;
  COMPLEX A, z, z1, z2;
  double d, e0, e, k, p;

  A =  csqrt(I * w * u * g);
  z = I * A * a;
  c.z1 = c.z2 = 0;
  z1 = 0.0;
  z2 = A0(a, A) / (PI * a * a * g) * Jn(0, z);
  // mutual inductance
  z2 += - I * w * U * 2.0 * log(a);
  z1 += - I * w * U * 12.0 * log(s);
  c.z1 += z1; c.z2 += z2;
  // proximity effect by outer conductors
  c1.z1 = c1.z2 = 0;
  for (n = 1; n <= 24; n++) {
	d = (double)n;
	z2 = pow(a / s, d) / d * Jn(n + 1, z) / z;
	c2 = Mn(n, a, s, A);
	c1.z1 += c2.z1 * z2; c1.z2 += c2.z2 * z2;
#ifdef PROFILE
	prof[2][n].c = c1; prof[2][n].n = n;
#endif
	e = cabs(c1.z1) + cabs(c1.z2);
	if (fabs((e - e0) / e) < 1e-4)
		break;
	e0 = e;
  }
  z1 = I * w * U1 * 12.0 * PI * a * a;
  c.z1 += c1.z1 * z1; c.z2 += c1.z2 * z1;
  return c;
}

// k = I2 / I1
// total current I = 6 * I1 + I2 == 1.0, k = I2 / I1

COMPLEX I2I1(double a, double s, double w, double u, double g)
{
  COMPLEX z;
  C2 c1, c2;

  c1 = Ea(a, s, w, u, g);
  c2 = Eg(a, s, w, u, g);
  z = (c2.z1 - c1.z1) / (c1.z2 - c2.z2);
  return z;
}

// un + I*vn = Jn(I*A*a) = Jn(b*I*sqrt(I) = bern(b) + I*bein(b)
// jn' = (d/db)(un)
// I*A*a = b*I*sqrt(I)

// (b / 2) * (u0*v0' - u0'*v0) / (u0'^2 + v0'^2)

COMPLEX UV0(double a, double w, double u, double g)
{
  double b, u0, v0, u1, v1;
  COMPLEX z, z0, z1;

  b = a * sqrt(w * u * g);
  z = b * I * csqrt(I);
  return 0.5 * b * I * csqrt(I) * Jn(0, z) / Jn(1, z);
}

// (b / (4 * w * U * g^2) * (un*vn' - un'*vn)

COMPLEX UVn(int n, double a, double w, double u, double g)
{
  double b, u0, v0, u1, v1;
  COMPLEX z, z0, z1;

  b = a * sqrt(w * u * g);
  z = b * I * csqrt(I);

  z0 = Jn(n, z);
  z1 = I * csqrt(I) * Jnd(n, z);
  return 0.25 * b / (w * U2 * g * g) * z0 * I * conj(z1);
}


// Impedance of 7 wire strand

COMPLEX Zac(double a, double s, double w, double u, double g)
{
  int n;
  COMPLEX A, I1, I2, k, z, z0, z1, z2, z3;
  C2 c;
  double e, x, x1, x2, y, y1, y2;

  k = I2I1(a, s, w, u, g);	// I2 / I1
  // calculated with I = 1.0 Ampare
  I1 = 1.0 / (6.0 + k);
  I2 = k / (6.0 + k);
  A = csqrt(I * w * u * g);
  // complex Poynting vector
  // outer conductor
  z1 = conj(I1) * I1 / (PI * a * a * g) * UV0(a, w, u, g);
  z0 = 0;
  for (n = 1; n <= 12; n++) {
	c = Mn(n, a, s, A);
	z3 = c.z1 * I1 + c.z2 * I2;	// Mn
	z1 += conj(z3) * z3 * UVn(n, a, w, u, g);
	if (n % 6 > 0)
		continue;
	e = cabs(z1 - z0);
	if (n > 1 && e < 1e-4)
		break;
	z0 = z1;
  }
  // center conductor
  z2 = conj(I2) * I2 / (PI * a * a * g) * UV0(a, w, u, g);
  z0 = 0;
  for (n = 1; n <= 24; n++) {
	c = Nn(n, a, s, A);
	z3 = c.z1 * I1 + c.z2 * I2;	// Nn
	z2 += conj(z3) * z3 * UVn(n, a, w, u, g);
	if (n % 6 > 0)
		continue;
	e = cabs(z2 - z0);
	if (n > 1 && e < 1e-4)
		break;
	z0 = z2;
  }
  return 6.0 * z1 + z2;
}

// AC Resistance of 7 wire strand
// I = I1 + I2 = 1.0

double Rac(double a, double s, double w, double u, double g)
{
  int n;
  COMPLEX A, I1, I2, k, z1, z2;
  C2 c;
  double e, x0, x, x1, x2;

  k = I2I1(a, s, w, u, g);	// I2 / I1
  // I = 6 * I1 + I2 = 1.0 Ampere
  I1 = 1.0 / (6.0 + k);
  I2 = k / (6.0 + k);
  A = csqrt(I * w * u * g);
  // loss I1
  x1 = conj(I1) * I1 / (PI * a * a * g) * creal(UV0(a, w, u, g));
  x0 = x = 0;
  for (n = 1; n <= 1; n++) {
	c = Mn(n, a, s, A);
	z1 = c.z1 * I1 + c.z2 * I2;	// Mn
	x += conj(z1) * z1 * creal(UVn(n, a, w, u, g));
	if (n % 6 > 0)
		continue;
	e = fabs(x - x0);
	if (n > 1 && e < 1e-4)
		break;
	x0 = x;
  }
  X3 = x;
  x1 += x;

  // loss I2
  x2 = conj(I2) * I2 / (PI * a * a * g) * creal(UV0(a, w, u, g));
  x0 = x = 0;
  for (n = 1; n <= 24; n++) {
	c = Nn(n, a, s, A);
	z1 = c.z1 * I1 + c.z2 * I2;	// Nn
	x += conj(z1) * z1 * creal(UVn(n, a, w, u, g));
	if (n % 6 > 0)
		continue;
	e = fabs(x - x0);
	if (n > 1 && e < 1e-4)
		break;
	x0 = x;
  }
  X4 += x;
  x2 += x;
  x = 6.0 * x1 + x2;
  return x;
}

// current density
//	*p1 = i1(r, t), *p2 = i2(r, t) .. 0<r<= a, 0<=t<=2*PI
int  idens(double a, double s, double g, double u, double f, double r, double t,
	COMPLEX *p1, COMPLEX *p2)
{
  int n;
  double b, w;
  COMPLEX A, I1, I2, z, z1;
  COMPLEX i1, i2;
  C2 c1, c2;

  w = 2.0 * PI * f;
  z1 = I2I1(a, s, w, u, g);
  I1 = 1.0 / (6.0 + z1);
  I2 = z1 / (6.0 + z1);

  b = a * sqrt(w * u * g);
  A = csqrt(I * w * u * g);
  z = I * A * a;

  i1 = i2 = (COMPLEX)0.0;
  i1 += I1 * A0(a, A);
  i2 += I2 * A0(a, A);
  for (n = 1; n <= 6; n++) {
	c1 = Mn(n, a, s, A);
	c2 = Nn(n, a, s, A);
	i1 += c1.z1 * Jn(n, z * a) * cos((double)n * t);
	i2 += c1.z2 * Jn(n, z * a) * cos((double)n * t);
	i1 += c2.z1 * Jn(n, z * a) * cos((double)n * t);
	i2 += c2.z2 * Jn(n, z * a) * cos((double)n * t);
  }
  i1 *= U3;
  i2 *= U3;
  *p1 = i1;
  *p2 = i2;
  return 0;
}


// Zac

int zac(double f1, double f2)
{
  double a, s, g, u, f, w, rdc;
  COMPLEX z, z1, z2;

  g = 5.80e7;
  u = 4e-7 * PI;
  a = 0.322e-3;	// 22AWG
  s = 1.0 * 2.0 * a;
//s = 1.05 * 2.0 * a;
//s = 1.88 * 2.0 * a;
//s = 3.85 * 2.0 * a;
//s = 6.32 * 2.0 * a;
//s = 9.88 * 2.0 * a;
  rdc = 1.0 / (PI * a * a * g) / 7.0;
  for (f = f1; f <= f2; f *= 1.05) {
	w = 2.0 * PI * f;
	z = Zac(a, s, w, u, g) / rdc;
	printf("%g %g %g\n", f, creal(z), cimag(z));
  }
  return 0;
}

int resitanceRatio(double f1, double f2)
{
  double a, s, g, u, f, w, rdc;
  COMPLEX z;

  g = 5.80e7;
  u = 4e-7 * PI;
  a = 0.322e-3;	// 22AWG
  s = 1.0 * 2.0 * a;
//s = 1.05 * 2.0 * a;
//s = 1.88 * 2.0 * a;
//s = 3.85 * 2.0 * a;
//s = 6.32 * 2.0 * a;
//  s = 9.88 * 2.0 * a;
  rdc = 1.0 / (PI * a * a * g) / 7.0;
  for (f = f1; f <= f2; f *= 1.05) {
	w = 2.0 * PI * f;
	z = Zac(a, s, w, u, g);
	printf("%g %g\n", f, creal(z) / rdc);
  }
  return 0;
}

int inductance(double f1, double f2)
{
  double a, s, g, u, f, w, rdc;
  COMPLEX z;

  g = 5.80e7;
  u = 4e-7 * PI;
  a = 0.322e-3;	// 22AWG
  s = 1.0 * 2.0 * a;
//s = 1.05 * 2.0 * a;
//s = 1.88 * 2.0 * a;
//s = 3.85 * 2.0 * a;
//s = 6.32 * 2.0 * a;
//s = 9.88 * 2.0 * a;
  rdc = 1.0 / (PI * a * a * g) / 7.0;
  for (f = f1; f <= f2; f *= 1.05) {
	w = 2.0 * PI * f;
	z = Zac(a, s, w, u, g);
	printf("%g %g\n", f, cimag(z) / w);
  }
  return 0;
}

Idensity(double a, double s, double g, double u, double f, double r)
{
  double  t;

  for (t = 0; t <= 2.0 * PI; t += PI * 0.01) {
	COMPLEX i1, i2;

	idens(a, s, g, u, f, r, t, &i1, &i2);
	printf("%g %g %g\n", t, cabs(i1), cabs(i2));
  }
  return 0;
}

int i2i1(double f1, double f2)
{
  double a, s, g, u, f, w, k;

  g = 5.80e7;
  u = 4e-7 * PI;
  a = 0.322e-3;	// 22AWG
  s = 1.0 * 2.0 * a;
//s = 1.05 * 2.0 * a;
//s = 1.88 * 2.0 * a;
//s = 3.85 * 2.0 * a;
//s = 6.32 * 2.0 * a;
//s = 9.88 * 2.0 * a;
  for (f = f1; f <= f2; f *= 1.05) {
	w = 2.0 * PI * f;
	k = I2I1(a, s, w, u, g);	// I2 / I1
	printf("%g %g %g\n", f, cabs(k), carg(k));
  }
  return 0;
}

// Solid conductor skin effect

COMPLEX Solid(double f, double a, double u, double g)
{
  double rdc, x;
  COMPLEX A, z;

  rdc = 1.0 / (g * PI * a * a);
  A = a * csqrt(I * 2.0 * PI * f * u * g);
  x = cabs(A);
  if (x < 1e-6)
	z = (COMPLEX)1.0;
  else if (5e2 < x)
	z = A * 0.5;
  else
	z = In(0, A) * (A / In(1, A)) * 0.5;
  return z * rdc;
}

int main()
{
  int i, n;
  double a, a1, a2, b, s, f, u, g, w, loss, loss1, loss2;
  double rdc, r1, r2, r, t;
  COMPLEX A, z, z1, z2;
  C2 c, c1, c2;

  PI = atan2(0.0, -1.0);

  zac(1e3, 1e8); exit(0);
//  resitanceRatio(1e3, 1e8); exit(0);
//  inductance(1e3, 1e8); exit(0);
//  i2i1(1e3, 1e8); exit(0);
  // cuurent density
  g = 5.80e7;
  u = 4e-7 * PI;
  a = 0.322e-3;	// 22AWG
  s = 1.0 * 2.0 * a;
//s = 1.05 * 2.0 * a;
//s = 1.88 * 2.0 * a;
//s = 3.85 * 2.0 * a;
//s = 6.32 * 2.0 * a;
//  s = 9.88 * 2.0 * a;
//  s = 1e1 * 2.0 * a;
//  s = 1.001 * 2.0 * a;
  Idensity(a, s, g, u, 1e5, a); exit(0);

  exit(0);
}

